declare module 'cpp' {
    namespace Cpp {
      export type Errors = {
        generate: typeof CppError.generate;
        CppError: typeof CppError;
        CppCardError: typeof CppCardError;
        CppInvalidRequestError: typeof CppInvalidRequestError;
        CppAPIError: typeof CppAPIError;
        CppAuthenticationError: typeof CppAuthenticationError;
        CppPermissionError: typeof CppPermissionError;
        CppRateLimitError: typeof CppRateLimitError;
        CppConnectionError: typeof CppConnectionError;
        CppSignatureVerificationError: typeof CppSignatureVerificationError;
        CppIdempotencyError: typeof CppIdempotencyError;
        CppInvalidGrantError: typeof CppInvalidGrantError;
      };
  
      export type RawErrorType =
        | 'card_error'
        | 'invalid_request_error'
        | 'api_error'
        | 'idempotency_error'
        | 'rate_limit_error'
        | 'authentication_error'
        | 'invalid_grant';
  
      export type CppRawError = {
        message?: string;
  
        type: RawErrorType;
  
        headers?: {[header: string]: string};
        statusCode?: number;
        requestId?: string;
  
        code?: string;
        doc_code?: string;
        decline_code?: string;
        param?: string;
        detail?: string;
  
        charge?: string;
        payment_intent?: Cpp.PaymentIntent;
        payment_method?: Cpp.PaymentMethod;
        setup_intent?: Cpp.SetupIntent;
        source?: Cpp.Source;
      };
  
      class CppError extends Error {
        constructor(rawError: CppRawError);
  
        static generate(
          rawError: CppRawError & {type: 'card_error'}
        ): CppCardError;
        static generate(
          rawError: CppRawError & {type: 'invalid_request_error'}
        ): CppInvalidRequestError;
        static generate(
          rawError: CppRawError & {type: 'api_error'}
        ): CppAPIError;
        static generate(
          rawError: CppRawError & {type: 'authentication_error'}
        ): CppAuthenticationError;
        static generate(
          rawError: CppRawError & {type: 'rate_limit_error'}
        ): CppRateLimitError;
        static generate(
          rawError: CppRawError & {type: 'idempotency_error'}
        ): CppIdempotencyError;
        static generate(
          rawError: CppRawError & {type: 'invalid_grant'}
        ): CppInvalidGrantError;
        static generate(
          rawError: CppRawError & {type: RawErrorType}
        ): CppError;
  
        /**
         * A human-readable message giving more details about the error. For card errors, these messages can
         * be shown to your users.
         */
        readonly message: string;
  
        readonly type: keyof Errors;
  
        /**
         * See the "error types" section at https://stripe.com/docs/api/errors
         */
        readonly rawType: RawErrorType;
  
        /**
         * For card errors, a short string describing the kind of card error that occurred.
         *
         * @docs https://stripe.com/docs/error-codes
         */
        readonly code?: string;
  
        /**
         * A URL to more information about the error code reported.
         *
         * @docs https://stripe.com/docs/error-codes
         */
        readonly doc_url?: string;
  
        /**
         * Typically a 4xx or 5xx.
         */
        readonly statusCode?: number;
  
        readonly raw: unknown;
  
        readonly headers: {
          [key: string]: string;
        };
  
        readonly requestId: string;
  
        /**
         * The parameter the error relates to if the error is parameter-specific. You can use this to display a
         * message near the correct form field, for example.
         */
        readonly param?: string;
  
        /** @deprecated */
        readonly detail?: unknown;
  
        readonly charge?: string;
        readonly decline_code?: string;
        readonly payment_intent?: PaymentIntent;
        readonly payment_method?: PaymentMethod;
        readonly payment_method_type?: string;
        readonly setup_intent?: SetupIntent;
        readonly source?: Source;
      }
  
      /**
       * Card errors are the most common type of error you should expect to handle.
       * They result when the user enters a card that can't be charged for some reason.
       */
      class CppCardError extends CppError {
        readonly type: 'CppCardError';
        readonly rawType: 'card_error';
  
        /**
         * @docs https://stripe.com/docs/declines/codes
         */
        readonly decline_code: string;
      }
  
      /**
       * Invalid request errors arise when your request has invalid parameters.
       */
      class CppInvalidRequestError extends CppError {
        readonly type: 'CppInvalidRequestError';
        readonly rawType: 'invalid_request_error';
      }
  
      /**
       * API errors cover any other type of problem (e.g., a temporary problem with Stripe's servers),
       * and are extremely uncommon.
       *
       * It could also be raised in the case that a new error has been introduced in the API,
       * but this version of the library doesn't know how to handle it.
       */
      class CppAPIError extends CppError {
        readonly type: 'CppAPIError';
        readonly rawType: 'api_error';
      }
  
      /**
       * Failure to properly authenticate yourself in the request.
       */
      class CppAuthenticationError extends CppError {
        readonly type: 'CppAuthenticationError';
        readonly rawType: 'authentication_error';
      }
  
      /**
       * Access was attempted on a resource that wasn't allowed.
       */
      class CppPermissionError extends CppError {
        readonly type: 'CppPermissionError';
      }
  
      /**
       * Too many requests hit the API too quickly.
       * @docs https://stripe.com/docs/rate-limits
       */
      class CppRateLimitError extends CppError {
        readonly type: 'CppRateLimitError';
        readonly rawType: 'rate_limit_error';
      }
  
      /**
       * The library cannot connect to Stripe.
       * This can happen for a variety of reasons,
       * such as loss of network connectivity or a bad TLS certificate.
       */
      class CppConnectionError extends CppError {
        readonly type: 'CppConnectionError';
      }
  
      /**
       * The signature verification for a webhook failed.
       * @docs https://stripe.com/docs/webhooks/signatures
       */
      class CppSignatureVerificationError extends CppError {
        readonly type: 'CppSignatureVerificationError';
      }
  
      /**
       * Idempotency errors occur when an `Idempotency-Key` is re-used on a request that does not match the first request's API endpoint and parameters.
       * @docs https://stripe.com/docs/api/idempotent_requests?lang=node
       */
      class CppIdempotencyError extends CppError {
        readonly type: 'CppIdempotencyError';
        readonly rawType: 'idempotency_error';
      }
  
      /**
       * InvalidGrantError is raised when a specified code doesn't exist, is
       * expired, has been used, or doesn't belong to you; a refresh token doesn't
       * exist, or doesn't belong to you; or if an API key's mode (live or test)
       * doesn't match the mode of a code or refresh token.
       */
      class CppInvalidGrantError extends CppError {
        readonly type: 'CppInvalidGrantError';
        readonly rawType: 'invalid_grant';
      }
    }
  }
  